/*
 * Object.cpp
 *
 *  Created on: 21.07.2009
 *      Author: ewert
 */

#include "Object.h"
#include <sstream>
namespace CCU
{

Object::Object()
{
}

Object::Object(std::string pName) :
	name(pName)
{
	//std::cout << pName << name << std::endl;
}

/*Object::Object(const Object* obj)
{
	this->setName(obj->getName());
	this->properties = obj->properties;
	this->relations = obj->relations;
}
*/

std::string Object::getName() const
{
	return name;
}

bool Object::hasProperty(std::string property) const
{
	std::set<std::string>::iterator it = properties.find(property);
	return (it == properties.end());
}

std::list<std::string> Object::getProperties() const
{
	std::list<std::string> tmp;
	std::set<std::string>::iterator it;
	for (it = properties.begin(); it != properties.end(); it++)
	{
		tmp.push_back(*it);
	}
	return tmp;
}

bool Object::addRelationTo(std::string name, Object *to)
{
	//tries to insert a new relation, and returns false if that fails
	std::pair<std::multimap<std::string, Object*>::const_iterator,
	std::multimap<std::string, Object*>::const_iterator> range;

	// check if already exists
	if(relations.size() > 0)
	{
		for(unsigned int i=0;i < relations.size(); i++)
		{
			if(relations.at(i) != name) continue;

			if(relatives.at(i) == to)
				return false;
		}
	}

	relations.push_back(name);
	relatives.push_back(to);

	return true;
}

void Object::setName(const std::string name)
{
	this->name = name;
}

void Object::removeRelationsTo(const Object* obj)
{
	/**
	 * achtung ... neuer stil!
	 * Nicht sicher ob Iteratoren oder direkt zugriff schneller ist ...
	 */

	for(unsigned int i=0;i< relatives.size() && i < relations.size(); i++)
	{
		if (relatives.at(i) == obj)
		{
			relatives.erase(relatives.begin()+i);
			relations.erase(relations.begin()+i);
		}
	}
}

bool Object::addProperty(std::string property)
{
	std::cout << this->name << " received property " << property << std::endl;
	return (properties.insert(property)).second;

}

std::list<std::string> Object::getRelationsTo(Object *to) const
{
	std::list<std::string> tmp;

	for(unsigned int i=0;i< relatives.size() && i < relations.size(); i++)
	{
		if (relatives.at(i) == to)
		{
			tmp.push_back(relations.at(i));
		}
	}

	return tmp;
}

void Object::removeProperty(const std::string prop)
{
	properties.erase(prop);
}

bool Object::hasRelation(std::string relation) const
{
	for(unsigned int i=0;i<relations.size();i++)
		if(relations.at(i) == relation)
			return true;

	return false;
}

std::list<std::string> Object::getRelatedToBy(std::string relation) const
{
	std::list<std::string> tmp;

	if(relations.size() > 0)
	{
		for(unsigned int i=0;i < relations.size() && i < relatives.size(); i++)
		{
			if(relations.at(i) != relation) continue;

			tmp.push_back(relatives.at(i)->getName());
		}
	}

	return tmp;

}
std::list<std::string> Object::getRelations() const
{
	std::list<std::string> tmp;

	for(unsigned int i=0;i < relations.size(); i++)
		tmp.push_back(relations.at(i));

	return tmp;
}

std::list<std::string> Object::getRelatedObjects() const
{
	std::list<std::string> tmp;

	for(unsigned int i=0;i<relatives.size();i++)
		tmp.push_back(relatives.at(i)->getName());

	return tmp;
}

std::string Object::toString() const
{
	std::stringstream tmp;

	tmp << "Name: ";
	tmp << this->getName();
	tmp << "\nProperties" << "(" << properties.size() << "):\n";

	for (std::set<std::string>::const_iterator it = properties.begin(); it
			!= properties.end(); ++it)
	{
		tmp << *it << "\n";
	}
	tmp << "Relations:\n";
	for (unsigned int i=0; i < relations.size() && i < relatives.size(); i++)
	{
		tmp << relations.at(i) << " " << (relatives.at(i))->getName() << "\n";
	}
	tmp << "\n";
	return tmp.str();
}

Object::Object(const Object & copy)
{

	//std::cout << "copying src:\n" << copy.toString() << std::endl;
	this->name = copy.name;
	this->properties = copy.properties;
	this->relations = copy.relations;
	//std::cout << "copy dest.:\n" << this->toString() << std::endl;
}

Object::~Object()
{
	// TODO Auto-generated destructor stub
}

// lenoch 14.10
bool Object::removeRelation(const Object* obj, const std::string type)
{
	for(unsigned int i=0;i < relations.size() && i < relatives.size(); i++)
	{
		if (relatives.at(i) == obj && (relations.at(i).compare(type) == 0))
		{
			relations.erase(relations.begin()+i);
			relatives.erase(relatives.begin()+i);
			return true;
		}
	}

	// Wasn't found
	return false;
}


}
