/*
 * State.cpp
 *
 *  Created on: 21.07.2009
 *      Author: ewert
 */

#include "State.h"

namespace CCU
{

State::State()
{
	// TODO Auto-generated constructor stub

}

State::~State()
{
	// TODO Auto-generated destructor stub
}

bool State::addObject(std::string object)
{
	// std::cout << "Received object to add: " << object << std::endl;
	Object tmp(object);

	// std::cout << "Temporary copy:" << tmp.toString() << std::endl;
	Object test = tmp;
	// std::cout << "Test copy:" << test.toString() << std::endl;

	return objects.insert(make_pair(object, /*Object(object)*/tmp)).second;
}

std::list<std::string> State::getObjectsWithRelation(std::string relation) const
{
	std::list<std::string> tmp;
	for (std::map<std::string, Object>::const_iterator it = objects.begin(); it
			!= objects.end(); ++it)
	{
		if ((it->second).hasRelation(relation))
			tmp.push_back(it->first);
	}
	return tmp;
}

std::list<std::string> State::getPropertiesOf(std::string object) const
{
	// Status : fixed and tested! (lenoch - 14.10)

	std::list<std::string> tmp;
	std::map<std::string, Object>::const_iterator it = objects.find(object);

	if (it != objects.end())
		/*std::list<std::string>*/ tmp = it->second.getProperties();

	return tmp;
}

bool State::addRelation(std::string from, std::string to, std::string relation)
{
	std::map<std::string, Object>::iterator from_it = objects.find(from);
	if (from_it == objects.end())
		return false;

	std::map<std::string, Object>::iterator to_it = objects.find(to);
	if (to_it == objects.end())
		return false;

	return from_it->second.addRelationTo(relation, &(to_it->second));
}

std::list<std::string> State::getObjectsWithProperty(std::string property) const
{
	std::list<std::string> tmp;
	for (std::map<std::string, Object>::const_iterator it = objects.begin(); it
			!= objects.end(); ++it)
	{
		if ((it->second).hasProperty(property))
			tmp.push_back(it->first);
	}
	return tmp;
}

std::list<std::string> State::getRelationsFrom(std::string object) const
{
	std::list<std::string> tmp;
	std::map<std::string, Object>::const_iterator it = objects.find(object);
	if (it == objects.end())
		return tmp;
	tmp = (it->second).getRelations();
	return tmp;
}

std::list<std::string> State::getRelatedObjects(std::string object) const
{
	 // Status: fixed and tested! - lenoch ...

	std::list<std::string> tmp;
	std::map<std::string, Object>::const_iterator it = objects.find(object);
	if (it != objects.end())
		/*std::list<std::string>*/ tmp = it->second.getRelatedObjects();
	return tmp;
}

std::list<std::string> State::getRelatedObjects(std::string object,
		std::string relation) const
{
	/*
	 * Siehe oben!!
	 *
	 * status: fixed and tested - lenoch 14.10
	 */
	std::list<std::string> tmp;
	std::map<std::string, Object>::const_iterator it = objects.find(object);
	if (it != objects.end())
		/*std::list<std::string>*/ tmp = it->second.getRelatedToBy(relation);
	return tmp;
}

std::list<std::string> State::getObjects() const
{
	std::list<std::string> tmp;
	std::map<std::string, Object>::const_iterator it;
	for (it = objects.begin(); it != objects.end(); it++)
	{
		tmp.push_back(it->first);
	}
	return tmp;
}

bool State::addProperty(std::string object, std::string property)
{
	if(objects.find(object)!=objects.end())
	{
		objects[object].addProperty(property);
		return true;
	}
	return false;
}

bool State::containsObject(std::string object) const
{

	std::map<std::string, Object>::const_iterator iter = objects.find(object);
	return (iter != objects.end());

}

std::string State::toString() const
{
	std::string tmp = "State contains:\n";
	for (std::map<std::string, Object>::const_iterator it = objects.begin(); it
			!= objects.end(); ++it)
	{
		tmp += (it->second).toString();
	}
	return tmp;
}

bool State::addState(const State &PosState)
{
	// --------------------------------------
	// Status:
	// Basic Tests done!
	// Everything seems to work correct!!
	// lenoch 14-10
	// --------------------------------------


	// 1) Add the new Objects
	// 2) Add Properties to New and Existing Objects
	// 3) Add Relations to New and Existing ...
	// ...

	// 1.) Add the new Objects ----------------------------------------------

	std::list<std::string> ObjList;
	std::list<std::string>::const_iterator itObj;
	std::map<std::string, Object>::const_iterator itTarget;

	ObjList = PosState.getObjects();

	for(itObj  = ObjList.begin();
		itObj != ObjList.end();
		itObj++)
	{
		// if the objects does not exist, add it!
		itTarget = objects.find(*itObj);

		if(itTarget == objects.end())
			addObject(*itObj);
	}

	// 2.) Add the new Properties ------------------------------------------
	std::list<std::string> PropList;
	std::list<std::string>::const_iterator itProp;

	for(itObj  = ObjList.begin();
		itObj != ObjList.end();
		itObj++)
	{
		// get the Properties
		PropList = PosState.getPropertiesOf(*itObj);

		for(itProp  = PropList.begin();
			itProp != PropList.end();
			itProp++)
		{
			// add the Properties
			// FIX: Sind die immer noch unikat?
			addProperty(*itObj, *itProp);
		}
	}

	// 3.) Add the new Relations -----------------------------------------
	std::list<std::string> RelObjList, RelTypList;
	std::list<std::string>::const_iterator itRelaObj, itRelaType;

	for(itObj  = ObjList.begin();
		itObj != ObjList.end();
		itObj++)
	{
		RelObjList = PosState.getRelatedObjects(*itObj);
		RelTypList = PosState.getRelationsFrom(*itObj);

		itRelaObj  = RelObjList.begin();
		itRelaType = RelTypList.begin();

		// get the Relations
		while((itRelaObj  != RelObjList.end()) &&
			  (itRelaType != RelTypList.end()))
		{
			// Add ...
			addRelation(*itObj, *itRelaObj, *itRelaType);

			// increment
			itRelaObj++;
			itRelaType++;
		}

	}

	return true;
}

bool State::removeState(const State &NegState)
{
	// ------------------------
	// Status: Tested!
	// Seems to work correct!!
	// lenoch - 14.10
	// ------------------------


	// If a Object is "lonely" in the NegState
	// it will delete the whole Object!!

	// 1.) check if an Object ist lonely
	// 2.) Delete the Properties
	// 3.) Delete the Relations
	// ------------------------------------------------------

	// 1.) Delete "lonely" Objects
	std::list<std::string> ObjList;
	std::list<std::string>::const_iterator itObj;

	ObjList = NegState.getObjects();

	for(itObj  = ObjList.begin();
		itObj != ObjList.end();
		itObj++)
	{
		if(NegState.getPropertiesOf(*itObj).size() == 0 &&
		   NegState.getRelatedObjects(*itObj).size() == 0)
		{
			// The State wants us to delete the Object!
			removeObject(*itObj);
		}
	}

	// 2.) Delete the Properties
	std::list<std::string> PropList;
	std::list<std::string>::const_iterator itProp;

	for(itObj  = ObjList.begin();
		itObj != ObjList.end();
		itObj++)
	{
		PropList = NegState.getPropertiesOf(*itObj);

		for(itProp  = PropList.begin();
			itProp != PropList.end();
			itProp++)
		{
			removeProperty(*itObj, *itProp);
		}
	}

	// 3.) Delete the Relations
	std::list<std::string> RelObjList, RelTypList;
	std::list<std::string>::const_iterator itRelaObj, itRelaType;

	for(itObj  = ObjList.begin();
		itObj != ObjList.end();
		itObj++)
	{
		RelObjList = NegState.getRelatedObjects(*itObj);
		RelTypList = NegState.getRelationsFrom(*itObj);

		itRelaObj  = RelObjList.begin();
		itRelaType = RelTypList.begin();

		// get the Relations
		while((itRelaObj  != RelObjList.end()) &&
			  (itRelaType != RelTypList.end()))
		{
			// Remove ...
			removeRelation(*itObj, *itRelaObj, *itRelaType);

			// increment
			itRelaObj++;
			itRelaType++;
		}

	}

	return true;
}


bool State::renameObject(std::string oldObject, std::string newObject)
{
	std::map<std::string, Object>::iterator it = objects.find(oldObject);
	if (it != objects.end())
	{
		Object temp = it->second;
		temp.setName(newObject);
		objects.erase(it);
		return objects.insert(make_pair(newObject, temp)).second;
	}
	return false;
}

// Function became useless!! 14.10
//std::list<std::string> State::getRelationsToObject(std::string from,
//		std::string to) const
//{
//	// find the object
//	std::list<std::string> tmp;
//	std::map<std::string, Object>::const_iterator it = objects.find(from);
//	std::map<std::string, Object>::const_iterator it_to = objects.find(to);
//	if (it == objects.end() || it_to == objects.end())
//		return tmp;
//
//	tmp = it->second.getRelationsTo((CCU::Object*) (&it->second));
//
//	return tmp;
//}

bool State::removeObject(const std::string object)
{
	// Test Status:
	// Basic Tests done!
	// "Remove Relations" Test Done! (see error in Object::removeRelationsTo)

	// find the object, than delete it
	std::map<std::string, Object>::iterator it;
	std::map<std::string, Object>::iterator target = objects.end();

	target = objects.find(object);

	if(target == objects.end())
		return true;

	// BUT, we also have to search through all
	// the other Objects, 'cos we have to destroy
	// the relations
	for(it  = objects.begin();
		it != objects.end();
		it++)
		it->second.removeRelationsTo(&target->second);

	// found it... now kill it
	objects.erase(target);

	return true;
}

bool State::removeRelation(const std::string from, const std::string to, const std::string type)
{
	// Test Status:
	// Basic Tests done!!

	// the Iterator
	std::map<std::string, Object>::iterator itFrom = objects.end();
	std::map<std::string, Object>::iterator itTo = objects.end();

	itFrom = objects.find(from);
	itTo   = objects.find(to);

	if(itFrom == objects.end() || itTo == objects.end())
		return true;

	// Just forward the request
	return itFrom->second.removeRelation(&itTo->second, type);
}

bool State::removeProperty(const std::string object, const std::string property)
{

	// Test Status:
	// Basic Tests Done!!

	// iterator
	std::map<std::string, Object>::iterator itObj = objects.end();

	// find ...
	itObj = objects.find(object);

	if(itObj == objects.end())
		return false;

	itObj->second.removeProperty(property);
	return true;
}

}

