#include <Ice/Ice.h>
#include <CCUIS.h>
#include <Workqueue.h>

using namespace std;

WorkQueue::WorkQueue() :
    _setcommanddone(false)
{
}

void
WorkQueue::run()
{
    IceUtil::Monitor<IceUtil::Mutex>::Lock lock(_monitor);

    while(!_setcommanddone)
    {
        if(_setcommandcallbacks.size() == 0)
        {
            _monitor.wait();
        }

        if(_setcommandcallbacks.size() != 0)
        {
            //
            // Get next work item.
            //
            setCommandCallbackEntry entry = _setcommandcallbacks.front();

            if(!_setcommanddone)
            {
                //
                // Print greeting and send response.
                //
            	Server::_commandChanged = false;
                _setcommandcallbacks.pop_front();
                Server::_CommandName = entry.name;
                for(LIST::const_iterator it = entry.parameter.begin(); it != entry.parameter.end(); ++it) {
                		Server::_CommandParameters.push_back(*it);
                }
                while(!Server::_commandChanged) {
                	;
                }
                entry.cb->ice_response();
            }
        }
    }

    //
    // Throw exception for any outstanding requests.
    //
    /*list<CallbackEntry>::const_iterator p;
    for(p = _callbacks.begin(); p != _callbacks.end(); ++p)
    {
        (*p).cb->ice_exception(Demo::RequestCanceledException());
    }*/
}

void
WorkQueue::add(const CCU::AMD_Crane_setCommandPtr& cb,const std::string& name,const CCU::LIST& parameter)
{
    IceUtil::Monitor<IceUtil::Mutex>::Lock lock(_monitor);

    if(!_setcommanddone)
    {
        //
        // Add work item.
        //
        setCommandCallbackEntry entry;
        entry.cb = cb;
        entry.name = name;
        for(CCU::LIST::const_iterator it = parameter.begin(); it != parameter.end(); ++it) {
        	entry.parameter.push_back(*it);
        }

        if(_setcommandcallbacks.size() == 0)
        {
            _monitor.notify();
        }
        _setcommandcallbacks.push_back(entry);
    }
    /*else
    {
        //
        // Destroyed, throw exception.
        //
        cb->ice_exception(Demo::RequestCanceledException());
    }*/
}

void
WorkQueue::destroy()
{
    IceUtil::Monitor<IceUtil::Mutex>::Lock lock(_monitor);

    //
    // Set done flag and notify.
    //
    _setcommanddone = true;
    _monitor.notify();
}
